#!/usr/bin/python
import argparse
import glob
import os
import pprint
import requests
import hashlib
import json
import subprocess
import urllib
import sys

embulk_command = "/root/.embulk/bin/embulk"
config_path = "/root/wwm_prod_onetime.yml.liquid"

command = ["sh", embulk_command,
           "-J-O",
           "run", config_path,
           "-L", "/root/embulk-parser-none-bin", "-L", "/home/eteri/embulk-input-filename", '-L' "/root/embulk-output-wendelin",
          ]

def check_files(base_path=None,
                file_expression=None,
                upload=False,
                delete_uploaded=False,
                url=None,
                info_url=None,
                tag="",
                user=None,
                password=None,
                start=None,
                end=None,
                file_size=None):

  not_uploaded_list = []
  diff_md5sum_list = []
  base_path = os.path.normpath(base_path) 
  if not url:
    reference = urllib.quote(os.path.join(tag, base_path, file_expression).replace('/', '.'))
    r = requests.get("%s?reference=%s" %(info_url, reference), auth=(user, password))
    url = str(r.text)
    print url

  if start is not None:
    start = os.path.join(base_path, start)
  if end is not None:
    end = os.path.join(base_path, end)    

  for path in glob.iglob(os.path.join(base_path, file_expression)):
    if start > path:
      continue
    if end is not None:
      if end < path:
        continue
    print(path)
    if file_size is not None:
      if os.stat(path).st_size != file_size:
        print("Not loaded because of size")
        continue
    if path.startswith(".") :   
      print("Not loaded because hidden")
      continue

    # replace i.e. 2017/10/17280145.BMR -> 2017.10.17280145.BMRa and url quote
    
    key = urllib.quote(path[len(base_path)+len(os.sep):].replace('/', '.'))
    print(key)

    r = requests.get("%s/hasBucketKey?key=%s" %(url, key), auth=(user, password))
    has_bucket_key_res = str(r.text)

    md5sum_local_file = hashlib.md5(open(path, 'rb').read()).hexdigest() # get md5 sum of the file_to_be_uploaded
    r = requests.get("%s/getMd5sum?key=%s" %(url, key), auth=(user, password))
    md5sum_uploaded_file = r.text  # get md5 sum of the uploaded file

    if has_bucket_key_res ==  'True' :
      if md5sum_local_file == md5sum_uploaded_file:
        if delete_uploaded is True:
          os.remove(path)
          print("UPLOADED and DELETED")
        else:
          print("UPLOADED")
      else:
        print("File is uploaded BUT md5sum is Different")
        diff_md5sum_list.append(path)
      print("local file  ", md5sum_local_file)
      print("uploaded file= ", md5sum_uploaded_file)
    else:
      print("NOT UPLOADED")
      not_uploaded_list.append(path)

  if upload is True:
    path_list = diff_md5sum_list + not_uploaded_list
    if path_list:
      subprocess.call(command, env={"path_list": json.dumps(path_list)})

  return sorted(not_uploaded_list), sorted(diff_md5sum_list)

def main(**kw):
  not_uploaded_list, diff_md5sum_list = check_files(**kw)

  print("number of not uploaded files : %s" %len(not_uploaded_list))
  print("list of not uploaded files: %s" %not_uploaded_list)
  print("number of uploaded files with different md5sum : %s" %len(diff_md5sum_list))
  print("list of uploaded files with different md5sum: %s" %diff_md5sum_list)

if __name__ == "__main__":
  # get the command line arguments
  parser = argparse.ArgumentParser(description='Test if all files are uploaded')
  parser.add_argument("-b", "--base_path", help = "Base path of the files to be uploaded, e.g. /mic/syscomtestuser/syscom/SYSCOM02-12400555/background", required = True, default = "")
  parser.add_argument("-f", "--file_expression", help = 'File expression of the files to be uploaded, e.g. "*/*/*.BMR"', required = True, default = "")
  parser.add_argument("-up","--upload", help = 'Upload missing files and files with different md5sum', action='store_true')
  parser.add_argument("-d", "--delete_uploaded", help = "Delete files on local file system which are uploaded an have same md5sum on server as locally", action='store_true')
  parser_mutex = parser.add_mutually_exclusive_group(required=True)
  parser_mutex.add_argument("-u", "--url", help = "Request url to data bucket stream, e.g. https://monitoring.woelfel.de/neo-erp5/data_stream_module/26", default = "")
  parser_mutex.add_argument("-i", "--info_url", help = "Request url to retrieve url of data bucket stream ", default = "")
  parser.add_argument("-t", "--tag", help = "Tag", required = True, default = "")
  parser.add_argument("-U", "--user", help = "Username", required = True, default = "")
  parser.add_argument("-P", "--password", help = "Password", required = True, default = "")
  parser.add_argument("-s", "--start", help = "First file to check", required = False, default = None)
  parser.add_argument("-e", "--end", help = "Last file to check", required = False, default = None)

  argument_dict = vars(parser.parse_args())
  main(**argument_dict)

